=== modified file 'NEWS'
--- NEWS	2008-06-10 07:51:09 +0000
+++ NEWS	2008-06-11 03:56:46 +0000
@@ -14,6 +14,10 @@
 
   BUGFIXES:
 
+    * You can now compare file revisions in Windows diff programs from 
+      Cygwin Bazaar.
+      (Matt McClure, #209281)
+
   DOCUMENTATION:
 
   TESTING:

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2008-05-08 04:33:38 +0000
+++ bzrlib/diff.py	2008-06-11 03:56:46 +0000
@@ -717,8 +717,8 @@
         return proc.wait()
 
     def _try_symlink_root(self, tree, prefix):
-        if not (getattr(tree, 'abspath', None) is not None
-                and osutils.has_symlinks()):
+        if (getattr(tree, 'abspath', None) is None
+            or not osutils.host_os_dereferences_symlinks()):
             return False
         try:
             os.symlink(tree.abspath(''), osutils.pathjoin(self._root, prefix))

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2008-05-22 12:53:10 +0000
+++ bzrlib/osutils.py	2008-06-11 03:56:46 +0000
@@ -848,6 +848,11 @@
         return False
 
 
+def host_os_dereferences_symlinks():
+    return (has_symlinks()
+            and sys.platform not in ('cygwin', 'win32'))
+
+
 def contains_whitespace(s):
     """True if there are any whitespace characters in s."""
     # string.whitespace can include '\xa0' in certain locales, because it is

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2008-06-06 13:30:30 +0000
+++ bzrlib/tests/test_diff.py	2008-06-11 03:56:46 +0000
@@ -19,6 +19,7 @@
 from cStringIO import StringIO
 import errno
 import subprocess
+import sys
 from tempfile import TemporaryFile
 
 from bzrlib import tests
@@ -41,6 +42,23 @@
                           TestCaseInTempDir, TestSkipped)
 
 
+class _AttribFeature(Feature):
+
+    def _probe(self):
+        if (sys.platform not in ('cygwin', 'win32')):
+            return False
+        try:
+            proc = subprocess.Popen(['attrib', '.'], stdout=subprocess.PIPE)
+        except OSError, e:
+            return False
+        return (0 == proc.wait())
+
+    def feature_name(self):
+        return 'attrib Windows command-line tool'
+
+AttribFeature = _AttribFeature()
+
+
 class _CompiledPatienceDiffFeature(Feature):
 
     def _probe(self):
@@ -1278,14 +1296,42 @@
         self.assertEqual('a-tool-which-is-unlikely-to-exist could not be found'
                          ' on this machine', str(e))
 
+    def test_prepare_files_creates_paths_readable_by_windows_tool(self):
+        self.requireFeature(AttribFeature)
+        output = StringIO()
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', 'content')])
+        tree.add('file', 'file-id')
+        tree.commit('old tree')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        diff_obj = DiffFromTool(['python', '-c',
+                                 'print "%(old_path)s %(new_path)s"'],
+                                tree, tree, output)
+        diff_obj._prepare_files('file-id', 'file', 'file')
+        self.assertReadableByAttrib(diff_obj._root, 'old\\file', r'old\\file')
+        self.assertReadableByAttrib(diff_obj._root, 'new\\file', r'new\\file')
+
+    def assertReadableByAttrib(self, cwd, relpath, regex):
+        proc = subprocess.Popen(['attrib', relpath],
+                                stdout=subprocess.PIPE,
+                                cwd=cwd)
+        proc.wait()
+        result = proc.stdout.read()
+        self.assertContainsRe(result, regex)
+
     def test_prepare_files(self):
         output = StringIO()
         tree = self.make_branch_and_tree('tree')
         self.build_tree_contents([('tree/oldname', 'oldcontent')])
+        self.build_tree_contents([('tree/oldname2', 'oldcontent2')])
         tree.add('oldname', 'file-id')
+        tree.add('oldname2', 'file2-id')
         tree.commit('old tree', timestamp=0)
         tree.rename_one('oldname', 'newname')
+        tree.rename_one('oldname2', 'newname2')
         self.build_tree_contents([('tree/newname', 'newcontent')])
+        self.build_tree_contents([('tree/newname2', 'newcontent2')])
         old_tree = tree.basis_tree()
         old_tree.lock_read()
         self.addCleanup(old_tree.unlock)
@@ -1303,7 +1349,7 @@
         self.assertContainsRe(new_path, 'new/newname$')
         self.assertFileEqual('oldcontent', old_path)
         self.assertFileEqual('newcontent', new_path)
-        if osutils.has_symlinks():
+        if osutils.host_os_dereferences_symlinks():
             self.assertTrue(os.path.samefile('tree/newname', new_path))
         # make sure we can create files with the same parent directories
-        diff_obj._prepare_files('file-id', 'oldname2', 'newname2')
+        diff_obj._prepare_files('file2-id', 'oldname2', 'newname2')

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2008-05-22 06:53:06 +0000
+++ bzrlib/tests/test_osutils.py	2008-06-11 03:56:46 +0000
@@ -322,6 +322,9 @@
         self.assertEqual("@", osutils.kind_marker("symlink"))
         self.assertRaises(errors.BzrError, osutils.kind_marker, "unknown")
 
+    def test_host_os_dereferences_symlinks(self):
+        osutils.host_os_dereferences_symlinks()
+
 
 class TestPumpFile(TestCase):
     """Test pumpfile method."""

